@@ -15,20 +15,27 @@ import sys
 import os
 
 VIMRC = '''
-" added by oh-my-vim
+" Added by oh-my-vim
 
-" path to oh-my-vim binary (take care of it if you are using a virtualenv)
-let g:ohmyvim="%(binary)s"
+" Change the default leader
+" let mapleader = ","
 
-" Use :OhMyVim profiles to list all available profiles
+" Skip upgrade of oh-my-vim itself during upgrades
+" let g:ohmyvim_skip_upgrade=1
+
+" Use :OhMyVim profiles to list all available profiles with a description
+" let profiles = %(profiles)r
 let profiles = ['defaults']
 
+" Path to oh-my-vim binary (take care of it if you are using a virtualenv)
+let g:ohmyvim="%(binary)s"
+
 " load oh-my-vim
 source %(ohmyvim)s
 
-" end of oh-my-vim required stuff
+" End of oh-my-vim required stuff
 
-" put your custom stuff bellow
+" Put your custom stuff bellow
 
 '''
 
@@ -115,7 +122,7 @@ class Bundle(object):
         if use_git:
             name = basename(url)[:-4]
             dirname = join(manager.runtime, name)
-            cmd = ['git', 'clone', '-q', url, dirname]
+            cmd = ['git', 'clone', '-q', '-b', 'master', url, dirname]
         elif use_hg:
             name = basename(url.strip('/'))
             dirname = join(manager.runtime, name)
@@ -133,25 +140,40 @@ class Bundle(object):
 
     def upgrade(self):
         self.log('Upgrading %s...', self.name)
-        if self.name == 'oh-my-vim':
+        if self.name.lower() == 'oh-my-vim':
             self.self_upgrade()
         else:
             os.chdir(self.dirname)
             if self.use_git:
-                p = Popen(['git', 'pull', '-qn'], stdout=PIPE)
+                p = Popen(['git', 'pull', '-qn', 'origin', 'master'],
+                           stdout=PIPE)
                 p.wait()
             elif self.use_hg:
                 p = Popen(['hg', 'pull', '-qu'], stdout=PIPE)
                 p.wait()
 
     def self_upgrade(self):
-        """Try to upgrade itself if a new version is available"""
+        """Try to upgrade itself"""
+        home = os.environ['HOME']
+
+        branch = "master"
+        with open(join(home, '.vimrc')) as fd:
+            for line in fd:
+                line = line.strip()
+                if not line.startswith('"'):
+                    if 'ohmyvim_skip_upgrade' in line and '=' in line:
+                        if int(line.split('=', 1)[1].strip()):
+                            line = line.replace('let', '').strip()
+                            self.log('Skipping. vimrc contains %s' % line)
+                            return
+                    elif 'ohmyvim_version' in line:
+                        branch = line.split('=').strip()
+
         if 'BUILDOUT_ORIGINAL_PYTHONPATH' in os.environ:
             self.log('Update your buildout then run:')
             self.log('    $ %s upgrade --force', sys.argv[0])
             return False
 
-        home = os.environ['HOME']
         install_dir = join(home, '.oh-my-vim/')
         if os.path.isdir(install_dir):
             bin_dir = join(install_dir, 'env', 'bin')
@@ -171,11 +193,13 @@ class Bundle(object):
                 cmd.append(('--install-option='
                             '--script-dir==%s') % bin_dir)
 
-            cmd.extend(['-e', 'git+%s@master#egg=oh-my-vim' % GIT_URL])
+            cmd.extend(['-e',
+                        'git+%s@%s#egg=oh-my-vim' % (self.remote, branch)])
 
             if home not in cmd[0]:
                 cmd.insert(0, 'sudo')
 
+            self.log('Upgrading to %s' % branch)
             if '__ohmyvim_test__' in os.environ:
                 self.log(' '.join(cmd))
                 return True
@@ -184,9 +208,12 @@ class Bundle(object):
                 p.wait()
                 return True
 
-        self.log('Dont know how to upgrade oh-my-vim...')
-        self.log('Update it manualy then run:')
-        self.log('    $ %s upgrade --force', sys.argv[0])
+        os.chdir(self.dirname)
+        p = Popen(['git', 'pull', '-qn', 'origin', branch], stdout=PIPE)
+        p.wait()
+
+        self.log("! Dont know how to upgrade oh-my-vim's python package...")
+        self.log('! You may try to update it manualy')
 
 
 class Manager(object):
@@ -230,18 +257,33 @@ class Manager(object):
                 fd.write('source %s\n' % join(self.ohmyvim, 'theme.vim'))
 
         if 'VIRTUAL_ENV' in os.environ:
-            binary = join(os.getenv('VIRTUAL_ENV'), 'oh-my-vim')
+            binary = join(os.getenv('VIRTUAL_ENV'), 'bin', 'oh-my-vim')
         else:
             binary = 'oh-my-vim'
-        kw = dict(ohmyvim=ohmyvim, binary=binary)
+
+        need_update = False
         if not isfile(self.vimrc):
-            with open(self.vimrc, 'w') as fd:
-                fd.write(VIMRC % kw)
+            need_update = True
         else:
             with open(self.vimrc) as fd:
                 if ohmyvim not in fd.read():
-                    with open(self.vimrc, 'a') as fd:
-                        fd.write(VIMRC % kw)
+                    need_update = True
+
+        if need_update:
+            if isfile(self.vimrc):
+                with open(self.vimrc, 'r') as fd:
+                    data = fd.read()
+                with open(join(self.ohmyvim, 'vimrc.bak'), 'w') as fd:
+                    fd.write(data)
+            else:
+                data = ''
+            kw = dict(
+                    ohmyvim=ohmyvim,
+                    binary=binary,
+                    profiles=self.profiles(None, as_list=True))
+            with open(self.vimrc, 'w') as fd:
+                fd.write(VIMRC % kw)
+                fd.write(data)
 
     def log(self, value, *args):
         if args:
@@ -310,22 +352,42 @@ class Manager(object):
 
     def install(self, args):
         config = get_config()
+        requires = join(os.path.dirname(__file__),
+                        '..', 'tools',
+                        'requires')
         if args.raw:
-            for name in sorted(config.bundles.keys()):
-                self.log(name)
-            for name in sorted(config.vimscripts.keys()):
-                self.log(name)
+            if args.dist:
+                for require in glob(join(requires, '*.txt')):
+                    name = basename(require)[:-4]
+                    if name != 'gawel':
+                        print name
+            else:
+                for name in sorted(config.bundles.keys()):
+                    self.log(name)
+                for name in sorted(config.vimscripts.keys()):
+                    self.log(name)
         else:
+            if args.dist:
+                if not isinstance(args.url, list):
+                    args.url = []
+                args.url.append(join(requires, '%s.txt', args.dist))
+
             dependencies = set()
             for url in args.url:
+                print url
                 if url.endswith('.txt'):
                     if isfile(url):
                         with open(url) as fd:
-                            dependencies = [d for d in fd.readlines()]
+                            for dep in fd:
+                                dep = dep.strip()
+                                if dep and not dep.startswith('#'):
+                                    dependencies.add(dep)
                     elif url.startswith('http'):
                         fd = urlopen(url)
-                        dependencies = dependencies.union(
-                                            set([d for d in fd.readlines()]))
+                        for dep in fd.readlines():
+                            dep = dep.strip()
+                            if dep and not dep.startswith('#'):
+                                dependencies.add(dep)
                 else:
                     b = Bundle.install(self, url)
                     if b:
@@ -369,10 +431,13 @@ class Manager(object):
                         self.log('* %s (%s)', b.name, b.remote)
                         self.log('\t- %s', ', '.join(themes))
 
-    def profiles(self, args):
+    def profiles(self, args, as_list=False):
         profiles = join(self.runtime, 'oh-my-vim', 'profiles')
         profiles = glob(join(profiles, '*.vim'))
 
+        if as_list:
+            return sorted([basename(name)[:-4] for name in profiles])
+
         for profile in sorted(profiles):
             name = basename(profile)[:-4]
             if not name.startswith('.'):
@@ -422,6 +487,8 @@ def main(*args):
 
     p = subparsers.add_parser('install', help='install a script or bundle')
     p.add_argument('--raw', action='store_true', default=False)
+    p.add_argument('-d', '--dist', default=None,
+                                   help="install a distribution")
     p.add_argument('url', nargs='*', default='')
     p.set_defaults(action=manager.install)
 
